#!/usr/bin/env python3

import sys
from html.parser import HTMLParser

class MyHTMLParser(HTMLParser):
	DE_TYPE_LIST = 1
	DE_TYPE_OBJECT = 2
	DE_TYPE_ENTRY = 3

	def __init__(self):
		self.check_type = False
		self.element = None
		self.token = None
		self.type = None
		self.obj_list = []
		self.ntabs = 0
		super().__init__()

	def handle_starttag(self, tag, attrs):
		if tag != 'td' :
			return

		if len(attrs) <= 1:
			return

		t, elem = attrs[1]
		if t == 'title' and elem.find('Device.WiFi.DataElements') >= 0:
			#print(elem)
			token_idx = elem.rfind('.')
			if  len(elem[token_idx+1:]) == 0:
				if elem[-5:] == '.{i}.':
					self.type = self.DE_TYPE_LIST
					token_idx = elem[0:-5].rfind('.')
					t = elem[token_idx+1:-5]
				elif elem[-1:] == '.':
					self.type = self.DE_TYPE_OBJECT
					token_idx = elem[0:-1].rfind('.')
					t = elem[token_idx+1:-1]
				else:
					raise('Unrecognized DE object')
			else:
				t = elem[token_idx+1:]
			#print(elem,  ' TOKEN: ', t)
			if elem.find('()') > 0:
				# Skip inputs
				return

			self.check_type = True
			self.element = elem
			self.token = t
		elif t == 'title' and self.check_type:
			#print(elem)
			self.check_type = False

			if len(self.obj_list) > 0:
				tmp_elem, tmp_token, tmp_type = self.obj_list[-1]
				'''
				# Check for duplicate
				if tmp_elem == self.element:
					# Duplicate element means input,
					# TODO better check for that, i.e. ()
					self.obj_list.pop()
					print(len(self.obj_list), "POP" , tmp_token)
					return;
				'''
				# Check for new object
				while self.element.find(tmp_elem) < 0 and len(self.obj_list) > 0:
					self.obj_list.pop()
					#print(len(self.obj_list), "POP" , tmp_token)
					tabs='\t' * len(self.obj_list)
					if tmp_type == self.DE_TYPE_LIST:
						print('%s}' % tabs)
						print(f'{tabs}blobmsg_close_array(bb, arr); // {tmp_token}')
					elif tmp_type == self.DE_TYPE_OBJECT:
						print(f'{tabs}blobmsg_close_table(bb, t); // {tmp_token}')
					else:
						raise('Not object or list')
					tmp_elem, tmp_token, tmp_type = self.obj_list[-1]

			tabs='\t' * len(self.obj_list)

			if elem == 'object':
				print(f'{tabs}/* {self.element} */');
				#print(len(self.obj_list), "ADD", elem, self.element, self.token)
				self.obj_list.append((self.element, self.token, self.type))

				if self.type == self.DE_TYPE_LIST:
					print(f'{tabs}arr = blobmsg_open_array(bb, "{self.token}");');
					print('%sfor (i = 0; i < 0 /* TODO */; i++) {' % tabs)
				elif self.type == self.DE_TYPE_OBJECT:
					print(f'{tabs}t = blobmsg_open_table(bb, "{self.token}");');
					#print("OBJECT")

			if (elem[0:len('string')] == 'string'):
				print(f'{tabs}blobmsg_add_string(bb, "{self.token}", "TODO");');
			elif (elem[0: len('unsignedInt')] == 'unsignedInt'):
				print(f'{tabs}blobmsg_add_u32(bb, "{self.token}", 0); // TODO {elem}');
			elif (elem[0: len('int')] == 'int'):
				print(f'{tabs}blobmsg_add_u32(bb, "{self.token}", 0); // TODO {elem}');
			elif (elem[0: len('unsignedLong')] == 'unsignedLong'):
				print(f'{tabs}blobmsg_add_u32(bb, "{self.token}", 0); // TODO {elem}');
			elif (elem == 'boolean'):
				print(f'{tabs}blobmsg_add_string(bb, "{self.token}", "TODO bool");');
			elif (elem == 'StatsCounter64'):
				print(f'{tabs}blobmsg_add_u64(bb, "{self.token}", 0); //TODO {elem}');
			elif (elem == 'StatsCounter32'):
				print(f'{tabs}blobmsg_add_u32(bb, "{self.token}", 0); //TODO {elem}');
			elif (elem == 'dateTime'):
				print(f'{tabs}blobmsg_add_string(bb, "{self.token}", "TODO {elem});');
			elif (elem == 'MACAddress'):
				print(f'{tabs}//hwaddr_ntoa(usta->macaddr, macaddr);)')
				print(f'{tabs}blobmsg_add_string(bb, "{self.token}", "TODO {elem}");');
			elif (elem == 'IPv4Address'):
				print(f'{tabs}blobmsg_add_string(bb, "{self.token}", "TODO {elem}");');
			elif (elem == 'IPv6Address'):
				print(f'{tabs}blobmsg_add_string(bb, "{self.token}", "TODO {elem}");');
			elif (elem[0:len('base64')] == 'base64'):
				# TODO add conversion to base64
				print(f'{tabs}blobmsg_add_string(bb, "{self.token}", "TODO {elem}");');
			elif (elem[0:len('hexBinary')] == 'hexBinary'):
				print(f'{tabs}blobmsg_add_string(bb, "{self.token}", "TODO {elem}");');
			elif (elem[0:len('list of')] == 'list of'):
				print(f'{tabs}arr = blobmsg_open_array(bb, "{self.token}");')
				print('%sfor (i = 0; i < 0 /* TODO */; i++) {' % tabs)
				tabs2 ='\t' * (len(self.obj_list) + 1)
				print(f'{tabs2}t = blobmsg_open_table(bb, "");')
				print(f'{tabs2} ; // {elem}')
				print(f'{tabs2}blobmsg_close_table(bb, t);')
				print('%s}'% tabs)
				print(f'{tabs}blobmsg_close_array(bb, arr);')
			elif (elem[0:len('list ')] == 'list'):
				print(f'{tabs}blobmsg_add_string(bb, "{self.token}", "TODO {elem}");');
			elif (elem == 'object'):
				pass
			else:
				print("Unknown type ", elem, "TOKEN", self.token)

	def finish(self):
		while len(self.obj_list) > 0:
			tmp_elem, tmp_token, tmp_type = self.obj_list.pop()
			#print(len(self.obj_list), "POP" , tmp_token)
			tabs='\t' * len(self.obj_list)
			if tmp_type == self.DE_TYPE_LIST:
				print('%s}' % tabs)
				print(f'{tabs}blobmsg_close_array(bb, arr); // {tmp_token}')
			elif tmp_type == self.DE_TYPE_OBJECT:
				print(f'{tabs}blobmsg_close_table(bb, t); // {tmp_token}')
			else:
				raise('Not object or list')

file_name = 'tr-181-2-15-0-usp.html'
if len(sys.argv) > 1:
	file_name = sys.argv[1]

parser = MyHTMLParser()
f = open(file_name, 'r')
txt = f.read()
parser.feed(txt)
parser.finish()

f.close()
